Package: tsna
Version: 0.3.5
Date: 2021-10-31
Title: Tools for Temporal Social Network Analysis
Type: Package
Authors@R: c(
           person("Skye", "Bender-deMoll", role=c("aut","cre"), email="skyebend@uw.edu"),
           person("Martina", "Morris", role=c("aut"), email="morrism@u.washington.edu"),
           person("James", "Moody", role=c("ctb"), email="jmoody77@soc.duke.edu")
           )
Depends: network (>= 1.13), networkDynamic (>= 0.9), R (>= 3.0)
Imports: statnet.common
Suggests: networkDynamicData, testthat, sna, knitr, rmarkdown,
        relevent, ergm (>= 3.10)
Description: Temporal SNA tools for continuous- and discrete-time longitudinal networks having vertex, edge, and attribute dynamics stored in the 'networkDynamic' format. This work was supported by grant R01HD68395 from the National Institute of Health.
License: GPL-3
LazyLoad: yes
URL: http://statnet.org/
BugReports: https://github.com/statnet/tsna/issues
VignetteBuilder: knitr
Author: Skye Bender-deMoll [aut, cre],
  Martina Morris [aut],
  James Moody [ctb]
Maintainer: Skye Bender-deMoll <skyebend@uw.edu>
Built: R 4.4.1; ; 2024-06-26 22:07:12 UTC; unix
